#!/usr/bin/env python

"""

    Whirlpool is a hash function designed by Vincent Rijmen 
    and Paulo Barreto that operates on messages less than 
    2^256 bits in length, and produces a message digest of 
    512 bits.
    
    Whirlpool has been selected for the NESSIE (EU) portfolio 
    of cryptographic primitives. The International Organization
    for Standardization (ISO) has decided to include the final 
    version of Whirlpool in the revised ISO/IEC 10118-3:2003(E)
    standard. 
    
    The Whirlpool algorithm has undergone two revisions since 
    its original 2000 specification (v0).  In the first revision 
    in 2001 (v1), the s-box was changed from a randomly 
    generated one with good cryptographic properties to one 
    which has better cryptographic properties and is easier to 
    implement in hardware.  Then, in 2003, the diffusion matrix 
    was changed in the second revision (v2).
    
    This is the 2003 version, aka v2.
    
    Whirlpool is not (and will never be) patented. It may be 
    used free of charge for any purpose. The reference 
    implementations are in the public domain.
    
    http://paginas.terra.com.br/informatica/paulobarreto/WhirlpoolPage.html
        
    This wrapper written by Larry Bugbee, May 2005
    rev August 2006

"""

from ctypes import cdll, create_string_buffer
import os

#---------------------------------------------------------------

def bytestr2hex(bs):
    h = ''
    for c in bs:
        h = h + ('0'+hex(ord(c))[2:])[-2:]
    return h

#---------------------------------------------------------------

# need to locate the .so file.  Assume it is in the same dir
# as this module
where = os.path.dirname(__file__)

if os.name == 'nt':
    wplib = cdll.LoadLibrary(os.path.join(where, "_whirlpool.pyd"))
else:
    wplib = cdll.LoadLibrary(os.path.join(where, "_whirlpool.so"))

class Whirlpool:

    def __init__(self):
        size = wplib.getNESSIEstructSize()      # should = 168
#        assert size == 168, 'Oops, size not right'
        self.ctx = create_string_buffer(size)
        wplib.NESSIEinit(self.ctx)

    def update(self, data):
        assert self.ctx != None, "digest already called"
        wplib.NESSIEadd(data, len(data)*8, self.ctx)

    def digest(self, data=''):
        assert self.ctx != None, "must NOT be called twice"
        if data:
            self.update(data)
        d = create_string_buffer(64)
        wplib.NESSIEfinalize(self.ctx, d)
        self.ctx = None       # we are done, kill ctx.  The C
                              #   pgm does not work after a
                              #   digest is fetched from ctx
        return d.raw

    def hexdigest(self, data=None):
        d = self.digest(data)
        return bytestr2hex(d)
    
    final = digest            # some hash classes use "final"

    # copy() won't work with this C pgm (I tried several 
    # themes and variations)

whirlpool = Whirlpool         # convenience, no cap required

def new():                    # alternate constructor, see sha
    return Whirlpool()
    
def digestsize():
    return 64
    
digest_size = digestsize      # both spellings supported by sha

#---------------------------------------------------------------
#---------------------------------------------------------------
#---------------------------------------------------------------
